package com.qmth.distributed.print.api;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.excel.ExcelField;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.entity.TCFinalScore;
import com.qmth.distributed.print.business.service.TCFinalScoreService;
import com.qmth.distributed.print.business.service.TCUsualScoreService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.enums.log.CustomizedOperationTypeEnum;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.IOException;
import java.util.*;

/**
 * <p>
 * 期末考试成绩表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Api(tags = "课程目标达成度-成绩管理-期末考试成绩Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_COURSE_DEGREE)
public class TCFinalScoreController {
    private final static Logger log = LoggerFactory.getLogger(TCFinalScoreController.class);

    @Resource
    TCUsualScoreService tcUsualScoreService;

    @Resource
    TCFinalScoreService tcFinalScoreService;

    @ApiOperation(value = "导入期末成绩-模板下载")
    @RequestMapping(value = "/final_score/template_download", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.EXPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "下载成功", response = Object.class)})
    public void finalScoreTemplateDownload(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                                           @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                                           @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) throws IOException {
        List<ExcelField> excelFieldList = new ArrayList<>();
        excelFieldList.add(new ExcelField("studentCode", "学号", true));
        excelFieldList.add(new ExcelField("name", "姓名", true));
        excelFieldList.add(new ExcelField("score", "成绩", true));
        excelFieldList.add(new ExcelField("key1", "1-1", true));
        excelFieldList.add(new ExcelField("key2", "1-2", false));
        tcUsualScoreService.downLoadExaminationTemplate("1、所有字段均为必填字段；\n" +
                "2、期末成绩按总分，和小题分录入，题号按大题号-小题号方式录入，如下，1-1表示第一大题第一小题，2-1表示第二大题第一小题；\n" +
                "3、请不要删除此行，也不要删除模板中的任何列；\n" +
                "4、使用前请先删除样例数据。", "sheet1", "期末成绩导入模版", excelFieldList);
    }

    @ApiOperation(value = "导入期末成绩")
    @RequestMapping(value = "/final_score/import", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.IMPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "导入成功", response = EditResult.class)})
    @Transactional
    public Object finalScoreExamImport(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file,
                                       @ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                                       @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                                       @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) throws IOException {
        log.debug("导入Excel开始...");
        long start = System.currentTimeMillis();
        MarkPaper markPaper = tcUsualScoreService.importExcelVaild(file, examId, paperNumber);

        Map<String, String> messageMap = new LinkedHashMap<>();
        try {
            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");
            List<Map<String, String>> list = EasyExcel.read(file.getInputStream()).headRowNumber(1).sheet(0).doReadSync();
            log.info("list:{}", JacksonUtil.parseJson(list));

            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            List<TCFinalScore> tcFinalScoreList = new ArrayList<>(list.size());
            Map<String, String> headMap = list.get(0);
            for (Map.Entry<String, String> entry : headMap.entrySet()) {
                if (Objects.equals(entry.getKey(), 0) && Objects.nonNull(entry.getValue()) &&
                        !Objects.equals(entry.getValue().trim(), "学号")) {
                    throw ExceptionResultEnum.ERROR.exception("excel表头第一行为学号");
                } else if (Objects.equals(entry.getKey(), 1) && Objects.nonNull(entry.getValue()) &&
                        !Objects.equals(entry.getValue().trim(), "姓名")) {
                    throw ExceptionResultEnum.ERROR.exception("excel表头第二行为姓名");
                } else if (Objects.equals(entry.getKey(), 2) && Objects.nonNull(entry.getValue()) &&
                        !Objects.equals(entry.getValue().trim(), "成绩")) {
                    throw ExceptionResultEnum.ERROR.exception("excel表头第三行为成绩");
                }
            }

            for (int i = 1; i < list.size(); i++) {
                Map<String, String> objectMap = list.get(i);
                boolean error = false;
                JSONArray jsonArray = new JSONArray();
                TCFinalScore tcFinalScore = new TCFinalScore(examId, courseCode, markPaper.getCourseName(), paperNumber, sysUser.getId());
                for (Map.Entry<String, String> entry : objectMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    if (Objects.isNull(entry.getValue()) || Objects.equals(entry.getValue().trim(), "")) {
                        errorData.add("excel第" + i + "行[").add(headMap.get(entry.getKey()) + "]为空;").add("\r\n");
                        error = true;
                    } else {
                        String head = headMap.get(entry.getKey());
                        Objects.requireNonNull(head, "表头信息异常");
                        if (Objects.equals(head.trim(), "学号")) {
                            tcFinalScore.setStudentCode(entry.getValue());
                        } else if (Objects.equals(head.trim(), "姓名")) {
                            tcFinalScore.setName(entry.getValue());
                        } else if (Objects.equals(head.trim(), "成绩")) {
                            tcFinalScore.setScore(Double.valueOf(entry.getValue()));
                        } else {
                            jsonObject.put("name", headMap.get(entry.getKey()));
                            jsonObject.put("score", entry.getValue());
                            jsonArray.add(jsonObject);
                        }
                    }
                }
                if (jsonArray.size() > 0) {
                    tcFinalScore.setScoreDetail(jsonArray.toJSONString());
                }
                if (!error) {
                    tcFinalScoreList.add(tcFinalScore);
                }
            }
            if (!CollectionUtils.isEmpty(tcFinalScoreList)) {
                successData.add("共导入" + tcFinalScoreList.size() + "条数据");

                QueryWrapper<TCFinalScore> tcScoreEndExamQueryWrapper = new QueryWrapper<>();
                tcScoreEndExamQueryWrapper.lambda().eq(TCFinalScore::getExamId, examId)
                        .eq(TCFinalScore::getCourseCode, courseCode)
                        .eq(TCFinalScore::getCourseName, markPaper.getCourseName())
                        .eq(TCFinalScore::getPaperNumber, paperNumber);
                tcFinalScoreService.remove(tcScoreEndExamQueryWrapper);
                tcFinalScoreService.saveBatch(tcFinalScoreList);
            }
            messageMap.put("正确信息", successData.length() > 0 ? successData.toString() : "无");
            messageMap.put("错误信息", errorData.length() > 0 ? errorData.toString() : "无");
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        log.debug("导入Excel结束...");
        long end = System.currentTimeMillis();
        log.info("============耗时{}秒============:", (end - start) / 1000);
        return ResultUtil.ok(messageMap);
    }

    @ApiOperation(value = "期末成绩列表")
    @RequestMapping(value = "/final_score/list", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = EditResult.class)})
    public Object finalScoreList(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                                 @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                                 @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                                 @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                 @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "期末成绩编辑")
    @RequestMapping(value = "/final_score/edit", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.EDIT)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = EditResult.class)})
    public Object finalScoreEdit(@ApiParam(value = "主键", required = true) @RequestParam Long id) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "期末成绩保存")
    @RequestMapping(value = "/final_score/save", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "保存", response = EditResult.class)})
    public Object finalScoreSave(@ApiParam(value = "主键", required = true) @RequestParam Long id,
                                 @ApiParam(value = "期末成绩", required = true) @RequestParam Double score,
                                 @ApiParam(value = "期末成绩明细", required = true) @RequestParam String scoreDetail) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "期末成绩启用/禁用")
    @RequestMapping(value = "/final_score/enable", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = EditResult.class)})
    public Object finalScoreEnable(@ApiParam(value = "主键", required = true) @RequestParam Long id,
                                   @ApiParam(value = "启用/禁用", required = true) @RequestParam Boolean enable) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "期末成绩试卷蓝图保存")
    @RequestMapping(value = "/final_score/paper_struct/save", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "试卷蓝图保存", response = Object.class)})
    public Object finalScorePaperStructSave(@ApiParam(value = "试卷蓝图结构", required = true) @RequestBody Object body) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "期末成绩试卷蓝图查询")
    @RequestMapping(value = "/final_score/paper_struct/query", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "试卷蓝图保存", response = Object.class)})
    public Object finalScorePaperStructQuery(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                                             @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                                             @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) throws IOException {
        return ResultUtil.ok(true);
    }
}
